@@ -1,4 +1,5 @@
 import datetime
+import hashlib
 import re
 
 from mopidy import backend
@@ -33,43 +34,70 @@ class BandcampLibraryProvider(backend.LibraryProvider):
                 ]
             if self.backend.config["bandcamp"]["identity"]:
                 dirs.append(Ref.directory(uri="bandcamp:collection", name="Collection"))
+                dirs.append(Ref.directory(uri="bandcamp:wishlist", name="Wishlist"))
             return dirs
-        if uri.startswith("bandcamp:collection"):
-            token = None
-            if uri != "bandcamp:collection":
-                token = uri[20:]
-            out = []
-            try:
-                data = self.backend.bandcamp.get_collection(token=token)
-                for i in data["items"]:
-                    art = f"a{i['art_id']:010d}" if ("art_id" in i) else None
-                    if i["tralbum_type"] == "a":
-                        aId = f"{i['band_id']}-{i['album_id']}"
-                        name = f"{i['band_name']} - {i['album_title']} (Album)"
-                        if art:
-                            self.images[aId] = art
-                        out.append(Ref.album(uri=f"bandcamp:myalbum:{aId}", name=name))
-                        self.scrape_urls[f"bandcamp:myalbum:{aId}"] = i["item_url"]
-                    elif i["tralbum_type"] == "t":
-                        aId = 0
-                        if i["album_id"] is not None:
-                            aId = i["album_id"]
-                        tId = f"{i['band_id']}-{aId}-{i['item_id']}"
-                        name = f"{i['item_title']} (Track)"
-                        if art:
-                            self.images[tId] = art
-                        out.append(Ref.album(uri=f"bandcamp:mytrack:{tId}", name=name))
-                        self.scrape_urls[f"bandcamp:mytrack:{tId}"] = i["item_url"]
-                if data["more_available"]:
-                    out.append(
-                        Ref.directory(
-                            uri="bandcamp:collection:" + data["last_token"],
-                            name="More...",
-                        )
+        for colltype in ["collection", "wishlist"]:
+            if uri.startswith("bandcamp:" + colltype):
+                token = None
+                if uri != "bandcamp:" + colltype:
+                    token = uri.split(":", 2)[2]
+                out = []
+                try:
+                    data = self.backend.bandcamp.get_collection(
+                        token=token, ctype=colltype
                     )
-            except Exception:
-                logger.exception("Failed to get collection")
-            return out
+                    for i in data["items"]:
+                        if "item_art" in i and "art_id" in i["item_art"]:
+                            art = (
+                                f"a{i['item_art']['art_id']:010d}"
+                                if i["item_art"]["art_id"]
+                                else None
+                            )
+                        if i["tralbum_type"] == "a":
+                            aId = f"{i['band_id']}-{i['album_id']}"
+                            name = f"{i['band_name']} - {i['album_title']} (Album)"
+                            if art:
+                                self.images[aId] = art
+                            if colltype == "collection":
+                                out.append(
+                                    Ref.album(uri=f"bandcamp:myalbum:{aId}", name=name)
+                                )
+                                self.scrape_urls[f"bandcamp:myalbum:{aId}"] = i[
+                                    "item_url"
+                                ]
+                            else:
+                                out.append(
+                                    Ref.album(uri=f"bandcamp:album:{aId}", name=name)
+                                )
+                        elif i["tralbum_type"] == "t":
+                            aId = 0
+                            if i["album_id"] is not None:
+                                aId = i["album_id"]
+                            tId = f"{i['band_id']}-{aId}-{i['item_id']}"
+                            name = f"{i['item_title']} (Track)"
+                            if art:
+                                self.images[tId] = art
+                            if colltype == "collection":
+                                out.append(
+                                    Ref.album(uri=f"bandcamp:mytrack:{tId}", name=name)
+                                )
+                                self.scrape_urls[f"bandcamp:mytrack:{tId}"] = i[
+                                    "item_url"
+                                ]
+                            else:
+                                out.append(
+                                    Ref.album(uri=f"bandcamp:track:{tId}", name=name)
+                                )
+                    if data["more_available"]:
+                        out.append(
+                            Ref.directory(
+                                uri="bandcamp:" + colltype + ":" + data["last_token"],
+                                name="More...",
+                            )
+                        )
+                except Exception:
+                    logger.exception("Failed to get collection")
+                return out
         if uri == "bandcamp:genres" or uri == "bandcamp:tags":
             stype = uri.split(":")[1]
             return [
@@ -128,22 +156,43 @@ class BandcampLibraryProvider(backend.LibraryProvider):
         ret = {}
         for uri in uris:
             ret[uri] = []
-            bcId = uri.split(":")[2]
-            if bcId in self.images:
-                i = self.images[bcId]
-                img = f"https://f4.bcbits.com/img/{i}"
+            component = uri.split(":")
+            if re.match(r"^(my)?(track|album|artist)", component[1], re.I):
+                bcId = uri.split(":")[2]
+                if bcId in self.images:
+                    i = self.images[bcId]
+                    img = f"https://f4.bcbits.com/img/{i}"
+                    for s in self.backend.image_sizes:
+                        if s in self.backend.bandcamp.IMAGE_SIZE:
+                            d = self.backend.bandcamp.IMAGE_SIZE[s]
+                            ret[uri].append(
+                                Image(uri=img + f"_{s}.jpg", width=d[0], height=d[1])
+                            )
+                        else:
+                            ret[uri].append(Image(uri=img + f"_{s}.jpg"))
+            else:
+                name = ""
+                if len(component) == 3:
+                    name = component[2]
+                elif len(component) == 2:
+                    name = component[1] if component[1] != "browse" else "bandcamp"
+                m = hashlib.md5(name.encode("utf-8")).hexdigest()
                 for s in self.backend.image_sizes:
                     if s in self.backend.bandcamp.IMAGE_SIZE:
                         d = self.backend.bandcamp.IMAGE_SIZE[s]
                         ret[uri].append(
-                            Image(uri=img + f"_{s}.jpg", width=d[0], height=d[1])
+                            Image(
+                                uri=f"https://dummyimage.com/{d[0]}x{d[1]}/{m[0:3]}/{m[3:6]}&text={quote(name)}",
+                                width=d[0],
+                                height=d[1],
+                            )
                         )
-                    else:
-                        ret[uri].append(Image(uri=img + f"_{s}.jpg"))
         return ret
 
     def lookup(self, uri):
         logger.debug('Bandcamp lookup "%s"', uri)
+        if len(uri.split(":")) != 3:
+            return []
         _, func, bcId = uri.split(":")
         ret = []
         if func == "album" or func == "track" or func.startswith("my"):
@@ -188,7 +237,10 @@ class BandcampLibraryProvider(backend.LibraryProvider):
             if "tags" in resp:
                 genre = "; ".join([t["name"] for t in resp["tags"]])
             elif "keywords" in resp:
-                genre = "; ".join(resp["keywords"].split(", "))
+                if type(resp["keywords"]) is list:
+                    genre = "; ".join(resp["keywords"])
+                else:
+                    genre = "; ".join(resp["keywords"].split(", "))
             artref = Artist(
                 uri=f"bandcamp:artist:{artist}",
                 name=resp["tralbum_artist"],
@@ -258,31 +310,39 @@ class BandcampLibraryProvider(backend.LibraryProvider):
             url = uri[9:]
             try:
                 resp = self.backend.bandcamp.scrape(url)
-                if 'tracks' not in resp:
-                    return(self.lookup(f"bandcamp:artist:{resp['id']}"))
+                if "tracks" not in resp:
+                    return self.lookup(f"bandcamp:artist:{resp['id']}")
                 else:
                     my = ""
-                    if 'is_purchased' in resp and resp['is_purchased']:
+                    if "is_purchased" in resp and resp["is_purchased"]:
                         my = "my"
-                    artist = resp['band_id']
+                    artist = resp["band_id"]
                     album = 0
-                    if resp['item_type'] == 'album':
-                        album = resp['id']
-                    elif 'current' in resp and 'album_id' in resp["current"]:
+                    if resp["item_type"] == "album":
+                        album = resp["id"]
+                    elif "current" in resp and "album_id" in resp["current"]:
                         album = resp["current"]["album_id"]
                     year = "0000"
                     if (
-                        "album_release_date" in resp and resp["album_release_date"] is not None
+                        "album_release_date" in resp
+                        and resp["album_release_date"] is not None
                     ):
                         year = resp["album_release_date"].split(" ")[2]
-                    elif "current" in resp and "release_date" in resp["current"] and resp["current"]["release_date"] is not None:
+                    elif (
+                        "current" in resp
+                        and "release_date" in resp["current"]
+                        and resp["current"]["release_date"] is not None
+                    ):
                         year = resp["current"]["release_date"].split(" ")[2]
                     comment = "URL: " + url
                     if "art_id" in resp:
                         self.images[f"{artist}-{album}"] = f"a{resp['art_id']:010d}"
                     genre = ""
                     if "keywords" in resp:
-                        genre = "; ".join(resp["keywords"].split(", "))
+                        if type(resp["keywords"]) is list:
+                            genre = "; ".join(resp["keywords"])
+                        else:
+                            genre = "; ".join(resp["keywords"].split(", "))
                     artref = Artist(
                         uri=f"bandcamp:artist:{artist}",
                         name=resp["tralbum_artist"],
@@ -329,7 +389,7 @@ class BandcampLibraryProvider(backend.LibraryProvider):
                                 ] = f"a{resp['art_id']:010d}"
                     logger.debug("Bandcamp returned %d tracks in lookup", len(ret))
             except Exception:
-                logger.error('Bandcamp failed to scrape "%s"', url)
+                logger.exception('Bandcamp failed to scrape "%s"', url)
         return ret
 
     def search(self, query=None, uris=None, exact=False):
